@@ -1,10 +1,14 @@
 import os.path
 import re
+from pathlib import Path
 from typing import TypedDict
 
+import toml
+import dunamai
 from betterproto.lib.google.protobuf.compiler import (
     CodeGeneratorRequest,
     CodeGeneratorResponse,
+    CodeGeneratorResponseFeature,
     CodeGeneratorResponseFile,
 )
 
@@ -49,6 +53,9 @@ def generate_code(request: CodeGeneratorRequest) -> CodeGeneratorResponse:
     Will look at the `gen_project` parameter if set to decide which
     file to use.
     """
+    response = CodeGeneratorResponse()
+    response.supported_features = CodeGeneratorResponseFeature.FEATURE_PROTO3_OPTIONAL
+
     params = parse_params(request.parameter)
 
     file_path = "pyproject.toml"
@@ -57,23 +64,36 @@ def generate_code(request: CodeGeneratorRequest) -> CodeGeneratorResponse:
         file_path = generate_pyproject_param["key"]
 
     if not os.path.exists(file_path):
-        return CodeGeneratorResponse(error=f"No project file found at '{file_path}'")
+        response.error = f"No project file found at '{file_path}'"
+        return response
 
     file_content = None
     with open(file=file_path) as f:
         file_content = f.read()
 
+    # Automatically set package version if configured to do so.
+    set_version_from_vcs = params.get("set_version_from_vcs")
+    if set_version_from_vcs is not None and set_version_from_vcs["key"] == "True":
+        pyproject_toml = toml.loads(file_content)
+        pyproject_toml["version"] = dunamai.Version.from_any_vcs().serialize()
+        file_content = toml.dumps(pyproject_toml)
+
     files = [CodeGeneratorResponseFile(name="pyproject.toml", content=file_content)]
 
-    include_py_typed = params.get("include_py_typed", True)
-    if include_py_typed is True:
+    include_py_typed = params.get("include_py_typed")
+    if include_py_typed is None or (
+        include_py_typed is not None and include_py_typed["key"] != "False"
+    ):
         package_name = params.get("package_name")
         if package_name is None:
-            return CodeGeneratorResponse(
-                error=f"package_name must be set if `include_py_typed` is True."
+            response.error = f"package_name must be set if `include_py_typed` is True."
+            return response
+
+        files.append(
+            CodeGeneratorResponseFile(
+                name=str(Path(package_name["key"]) / "py.typed"), content=""
             )
+        )
 
-        files.append(CodeGeneratorResponseFile(name="py.typed", content=""))
-
-    response = CodeGeneratorResponse(file=files)
+    response.file = files
     return response
